pub enum Error {
ParseError {
path: String,
message: String,
},
IoError {
kind: String,
message: String,
},
UnsupportedMimeType {
mime_type: String,
},
MissingContentType,
Other {
message: String,
},
FailureToNegotiateContent,
}Expand description
A serde-serializable error
Variants
ParseError
An error occurred in parsing the provided body content
Fields

path: Stringthe path of the parse error, as provided by serde_path_to_error
IoError
A transmission error occurred in the connection to the http client
UnsupportedMimeType
The client provided a content type that this library does not yet support
MissingContentType
The client did not provide a content-type
Other
Miscellaneous other errors – please open an issue on trillium-api if you find yourself parsing the contents of this.
FailureToNegotiateContent
we were unable to find a content type that matches the Accept header. Please open an issue if you’d like an additional format to be supported