pub struct BeforeSend<H>(pub H);Expand description
Run a [trillium::Handler] in before_send
Wrap a handler with this type to call its run callback in before_send allowing it to execute
even if the conn has been halted. Note that it will be called later in the handler sequence as a
result of this.
Tuple Fields
0: HTrait Implementations
Sourceimpl<H: Debug> Debug for BeforeSend<H>
impl<H: Debug> Debug for BeforeSend<H>
Sourceimpl<H> Handler for BeforeSend<H>
impl<H> Handler for BeforeSend<H>
Sourceasync fn run(&self, conn: Conn) -> Conn
async fn run(&self, conn: Conn) -> Conn
Executes this handler, performing any modifications to the Conn that are desired.
Sourceasync fn init(&mut self, info: &mut Info)
async fn init(&mut self, info: &mut Info)
Performs one-time async set up on a mutable borrow of the Handler before the server starts
accepting requests. This allows a Handler to be defined in synchronous code but perform
async setup such as establishing a database connection or fetching some state from an
external source. This is optional, and chances are high that you do not need this. Read more
Sourceasync fn before_send(&self, conn: Conn) -> Conn
async fn before_send(&self, conn: Conn) -> Conn
Performs any final modifications to this conn after all handlers have been run. Although
this is a slight deviation from the simple conn->conn->conn chain represented by most
Handlers, it provides an easy way for libraries to effectively inject a second handler into
a response chain. This is useful for loggers that need to record information both before and
after other handlers have run, as well as database transaction handlers and similar library
code. Read more
Sourcefn name(&self) -> Cow<'static, str>
fn name(&self) -> Cow<'static, str>
Customize the name of your handler. This is used in Debug implementations. The default is
the type name of this handler.
Sourcefn has_upgrade(&self, upgrade: &Upgrade) -> bool
fn has_upgrade(&self, upgrade: &Upgrade) -> bool
predicate function answering the question of whether this Handler would like to take
ownership of the negotiated Upgrade. If this returns true, you must implement
[
Handler::upgrade]. The first handler that responds true to this will receive
ownership of the [trillium::Upgrade][crate::Upgrade] in a subsequent call to
[Handler::upgrade]Sourceasync fn upgrade(&self, upgrade: Upgrade)
async fn upgrade(&self, upgrade: Upgrade)
This will only be called if the handler reponds true to [
Handler::has_upgrade] and will
only be called once for this upgrade. There is no return value, and this function takes
exclusive ownership of the underlying transport once this is called. You can downcast
the transport to whatever the source transport type is and perform any non-http protocol
communication that has been negotiated. You probably don’t want this unless you’re
implementing something like websockets. Please note that for many transports such as
TcpStreams, dropping the transport (and therefore the Upgrade) will hang up /
disconnect.Auto Trait Implementations
impl<H> Freeze for BeforeSend<H>where
H: Freeze,
impl<H> RefUnwindSafe for BeforeSend<H>where
H: RefUnwindSafe,
impl<H> Send for BeforeSend<H>where
H: Send,
impl<H> Sync for BeforeSend<H>where
H: Sync,
impl<H> Unpin for BeforeSend<H>where
H: Unpin,
impl<H> UnsafeUnpin for BeforeSend<H>where
H: UnsafeUnpin,
impl<H> UnwindSafe for BeforeSend<H>where
H: UnwindSafe,