pub struct CancelOnDisconnect<F, OutputHandler, TryFromConn>(/* private fields */);Expand description
A struct that cancels a handler if the client disconnects.
Note that the conn is not available to this handler, and any properties of the request needed
for execution must be extracted through FromConn or
TryFromConn arguments
Implementations